@@ -1,38 +1,10 @@
 #!/usr/bin/python3
-# -*- coding: utf-8 -*-
 #
 # Univention Management Console
 #  module: password reset service
 #
-# Like what you see? Join us!
-# https://www.univention.com/about-us/careers/vacancies/
-#
-# Copyright 2015-2024 Univention GmbH
-#
-# https://www.univention.de/
-#
-# All rights reserved.
-#
-# The source code of this program is made available
-# under the terms of the GNU Affero General Public License version 3
-# (GNU AGPL V3) as published by the Free Software Foundation.
-#
-# Binary versions of this program provided by Univention to you as
-# well as other copyrighted, protected or trademarked materials like
-# Logos, graphics, fonts, specific documentations and configurations,
-# cryptographic keys etc. are subject to a license agreement between
-# you and Univention and not subject to the GNU AGPL V3.
-#
-# In the case you use this program under the terms of the GNU AGPL V3,
-# the program is provided in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
-# GNU Affero General Public License for more details.
-#
-# You should have received a copy of the GNU Affero General Public
-# License with the Debian GNU/Linux or Univention distribution in file
-# /usr/share/common-licenses/AGPL-3; if not, see
-# <https://www.gnu.org/licenses/>.
+# SPDX-FileCopyrightText: 2015-2025 Univention GmbH
+# SPDX-License-Identifier: AGPL-3.0-only
 
 import datetime
 import email.charset
@@ -45,7 +17,7 @@ from email.mime.nonmultipart import MIMENonMultipart
 from email.utils import formatdate
 from functools import wraps
 from subprocess import PIPE, STDOUT, Popen
-from typing import Any, Dict, List
+from typing import Any
 
 import atexit
 import pylibmc
@@ -56,7 +28,7 @@ import univention.admin.syntax
 import univention.admin.uexceptions as udm_errors
 from univention.admin.uldap import getMachineConnection
 from univention.lib.i18n import Translation
-from univention.lib.umc import Client, ConnectionError, HTTPError, Unauthorized
+from univention.lib.umc import Client, ConnectionError, HTTPError, Unauthorized  # noqa: A004
 from univention.management.console.config import ucr
 from univention.management.console.ldap import (
     get_admin_connection, get_machine_connection, get_user_connection, machine_connection,
@@ -160,6 +132,22 @@ def prevent_denial_of_service(func):
 
     @wraps(func)
     def _decorated(self, *args, **kwargs):
+        request = self._current_request
+
+        client_host_str = None
+        # Determine the client host. Respect X-Forwarded-Host if configured and present.
+        x_forwarded_host = request.headers.get('X-Forwarded-Host')
+        if x_forwarded_host:
+            # Take the first Host in the list (closest client)
+            client_host_str = x_forwarded_host.split(',')[0].strip()
+            MODULE.debug("Rate limit check: Using X-Forwarded-Host: %s", client_host_str)
+        else:
+            MODULE.debug("Rate limit check: Cannot determine remote Host from request object.")
+
+        if client_host_str and client_host_str in self.trusted_hosts:
+            MODULE.debug("Rate limit bypassed for trusted host %s in trusted hosts %r" % (client_host_str, self.trusted_hosts))
+            return func(self, *args, **kwargs)
+
         # check total request limits
         total_limit_reached, total_max_wait = _check_limits(self.memcache, self.total_limits)
 
@@ -199,7 +187,7 @@ class ConnectionLimitReached(UMC_Error):
     status = 503
 
     def __init__(self, seconds):
-        super(ConnectionLimitReached, self).__init__(_("The allowed maximum number of connections to the server has been reached. Please retry in {}.").format(seconds))
+        super().__init__(_("The allowed maximum number of connections to the server has been reached. Please retry in {}.").format(seconds))
 
 
 class ServiceForbidden(UMC_Error):
@@ -207,14 +195,14 @@ class ServiceForbidden(UMC_Error):
     status = 403
 
     def __init__(self):
-        super(ServiceForbidden, self).__init__(_("Either username or password is incorrect or you are not allowed to use this service."))
+        super().__init__(_("Either username or password is incorrect or you are not allowed to use this service."))
 
 
 class TokenNotFound(UMC_Error):
     status = 400
 
     def __init__(self):
-        super(TokenNotFound, self).__init__(
+        super().__init__(
             _("The token you supplied is either expired or invalid. Please request a new one."))
 
 
@@ -222,7 +210,7 @@ class NoMethodsAvailable(UMC_Error):
     status = 403
 
     def __init__(self):
-        super(NoMethodsAvailable, self).__init__(_('No contact information is stored for this user. Resetting the password is not possible.'))
+        super().__init__(_('No contact information is stored for this user. Resetting the password is not possible.'))
 
 
 class TokenSendMessage(UMC_Error):
@@ -241,6 +229,13 @@ class Instance(Base):
         self._usersmod = None
         self.groupmod = None
 
+        self.trusted_hosts = [
+            h.strip()
+            for h in ucr.get('umc/self-service/rate-limit/trusted-hosts', '').split(',')
+            if h.strip()
+        ]
+        MODULE.info("Added trusted host for rate limit bypass: %r", self.trusted_hosts)
+
         self.token_validity_period = ucr.get_int("umc/self-service/passwordreset/token_validity_period", 3600)
         limit_total_minute = ucr.get_int("umc/self-service/passwordreset/limit/total/minute", 0)
         limit_total_hour = ucr.get_int("umc/self-service/passwordreset/limit/total/hour", 0)
@@ -256,7 +251,6 @@ class Instance(Base):
         ]
         if IS_SELFSERVICE_MASTER:
             self.db = TokenDB(MODULE)
-            self.conn = self.db.conn
             atexit.register(self.db.close_db)
             if not self.db.table_exists():
                 self.db.create_table()
@@ -386,7 +380,7 @@ class Instance(Base):
             dict(wd, value=properties.get(wd['id'])) for wd in self._get_user_attributes_descriptions()
             if user.has_property(wd['id'])
         ]
-        # TODO make layout configurable via ucr ?
+        # TODO: make layout configurable via ucr ?
         layout = [wd['id'] for wd in widget_descriptions]
 
         return {
@@ -517,7 +511,7 @@ class Instance(Base):
 
             isValid = True
             message = ''
-            if prop.multivalue and isinstance(value, (tuple, list)):
+            if prop.multivalue and isinstance(value, tuple | list):
                 isValid = []
                 message = []
                 for ival in value:
@@ -616,7 +610,7 @@ class Instance(Base):
                     'message': _('The username is already taken'),
                 }
         invalid = {k: v for (k, v) in res.items() if not (all(v['isValid']) if isinstance(v['isValid'], list) else v['isValid'])}
-        if len(invalid):
+        if invalid:
             return {
                 'success': False,
                 'failType': 'INVALID_ATTRIBUTES',
@@ -685,7 +679,7 @@ class Instance(Base):
         try:
             new_user.create()
         except univention.admin.uexceptions.base as exc:
-            password_complexity_message = self._get_password_complexity_message() if isinstance(exc, (udm_errors.pwToShort, udm_errors.pwQuality)) else ''
+            password_complexity_message = self._get_password_complexity_message() if isinstance(exc, udm_errors.pwToShort | udm_errors.pwQuality) else ''
             MODULE.error('create_self_registered_account(): could not create user: %s' % (exc,))
             return {
                 'success': False,
@@ -693,7 +687,7 @@ class Instance(Base):
                 'data': (_('The account could not be created:\n%s\n%s') % (UDM_Error(exc), password_complexity_message)).rstrip(),
             }
         finally:
-            # TODO cleanup
+            # TODO: cleanup
             # reinit user module without template.
             # This has to be done since the modules are singletons?
             univention.admin.modules.update()
@@ -997,7 +991,7 @@ class Instance(Base):
 
     @forward_to_master
     @simple_response
-    def get_reset_methods(self) -> List[Dict[str, Any]]:
+    def get_reset_methods(self) -> list[dict[str, Any]]:
         if ucr.is_false('umc/self-service/passwordreset/backend/enabled'):
             msg = _('The password reset was disabled via the Univention Configuration Registry.')
             MODULE.error(f'get_reset_methods(): {msg}')
@@ -1040,8 +1034,7 @@ class Instance(Base):
         try:
             self._call_send_msg_plugin(username, method, address, token, user_properties)
         except Exception:
-            MODULE.error("send_token(): Error sending token with via '{method}' to '{username}'.".format(
-                method=method, username=username))
+            MODULE.error(f"send_token(): Error sending token with via '{method}' to '{username}'.")
             self.db.delete_tokens(username=username)
             raise
         return True
@@ -1157,7 +1150,7 @@ class Instance(Base):
             reset_username = dict(ucr)['ad/reset/username']
             with open(dict(ucr)['ad/reset/password']) as fd:
                 reset_password = fd.readline().strip()
-        except (EnvironmentError, KeyError):
+        except (OSError, KeyError):
             raise UMC_Error(_('The configuration of the password reset service is not complete. The UCR variables "ad/reset/username" and "ad/reset/password" need to be set properly. Please inform an administrator.'), status=500)
         process = Popen(['samba-tool', 'user', 'setpassword', '--username', reset_username, '--password', reset_password, '--filter', filter_format('samaccountname=%s', (username,)), '--newpassword', password, '-H', ldb_url], stdout=PIPE, stderr=STDOUT)
         stdouterr = process.communicate()[0].decode('utf-8', 'replace')
